////////////////////////////////////////////////////////////////////////////////
/// -Vendaric MAP FILE- ////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

addMaterialMapping("terrain/ScorpDirt05", "color: 0.46 0.36 0.26 0.4 0.0", "sound: 0");

package Vendaric
{
   function Vendaric::preLoad(%firstMission)
   {
      // Need to store the map name for scripts to compare to current mission name
      $Map::MyMap = "Vendaric";

      // For our undeploy code, we don't have to reset these at map end
      DeployedStationInventory.item = InventoryDeployable;
      DeployedMotionSensor.item = MotionSensorDeployable;
      DeployedPulseSensor.item = PulseSensorDeployable;
      TurretDeployedFloorIndoor.item = TurretIndoorDeployable;
      TurretDeployedWallIndoor.item = TurretIndoorDeployable;
      TurretDeployedCeilingIndoor.item = TurretIndoorDeployable;
      TurretDeployedOutdoor.item = TurretOutdoorDeployable;
   }

   function Vendaric::InitMap()
   {
      for(%i = 0; %i <= %game.numTeams; %i++)
         $ObjectiveCounter[%i] = 0;

      %pzGroup = nameToID("MissionCleanup/PZones");
      for(%i = 0; %i < %pzGroup.getCount(); %i++)
      {
         %pz = %pzGroup.getObject(%i);
         %pz.velocityMod = 1;
         %pz.gravityMod = 0;
      }

      orgRandom("PhoenixPlant1", 30, 1000, "0 0 164");
      orgRandom("PhoenixPlant2", 30, 1000, "0 0 164");
      orgRandom("PhoenixPlant5", 30, 1000, "0 0 164");

      // Bug in DnD that adds missionTypesList objects to objectives.
      // So we have to manually remove them from the objectives.
      // Timing is very tight here as this is called in the same function
      // that DnD sets up objectives in. (DefaultGame::missionLoadDone)
      if(Game.class $= "DnDGame")
      {
         %group = nameToID("MissionGroup/Teams");
         %group.cleanDnDObjectives();
      }
   }

   function Cadeilia::DeactivateMap()
   {
      // Remove all of our store varibles from memory
      deleteVariables("$Map::*");

      // Must deactivate package, may be a local game, also failsafe
      if(isActivePackage($MissionName))
      {
         deactivatePackage($MissionName);
         echo("Deactivating package: " @ $MissionName);
      }
   }

   function SimGroup::cleanDnDObjectives(%this)
   {
      Game.cleanupObjectives(%this);
   }

   function DnDGame::cleanupObjectives(%game, %this)
   {
      for(%i = 0; %i < %this.getCount(); %i++)
      {
         %obj = %this.getObject(%i);
         if(%obj.getClassName() $= SimGroup)
            %obj.cleanDnDObjectives();

         for(%j = 0; (%typeList = getWord(%obj.missionTypesList, %j)) !$= ""; %j++)
            if(%typeList $= DnD)
               %found = 1;

         if(!%found) // Ok so we didn't find DnD in there..
         {
            if(%obj.missionTypesList !$= "")// && %obj.missionTypesList !$= "DnD")
            {
               if(isObject(%obj.trigger)) // For this map we have to clean some triggers up..
                  %obj.trigger.delete();

               if(%obj.scoreValue == true && !%obj.objectiveCompleted)
               {
                  error("Found non-DnD " @ %obj.getDataBlock().getName() @ " objective on map, removing from objectives..");
                  %obj.scoreValue = false;
                  %obj.objectiveCompleted = true;
                  $numObjectives[1]--;
                  $numObjectives[2]--;
               }
            }
         }
      }
   }

   function CTFGame::createPlayer(%game, %client, %spawnLoc, %respawn)
   {
      DefaultGame::createPlayer(%game, %client, %spawnLoc, %respawn);
      if(strlwr(getModPaths()) $="classic;base")
      {
         %player = %client.player;
         if(%client.isAiControlled())
            %player.setRechargeRate(%player.getDataBlock().rechargeRate + 0.15);
      }
   }

   function DnDGame::createPlayer(%game, %client, %spawnLoc, %respawn)
   {
      DefaultGame::createPlayer(%game, %client, %spawnLoc, %respawn);
      if(strlwr(getModPaths()) $="classic;base")
      {
         %player = %client.player;
         if(%client.isAiControlled())
            %player.setRechargeRate(%player.getDataBlock().rechargeRate + 0.15);
      }
   }

   function Armor::onEnterLiquid(%data, %obj, %coverage, %type)
   {
      if(%obj.client.isAiControlled())
         return;
      else
         Parent::onEnterLiquid(%data, %obj, %coverage, %type);
   }

   function ShapeBaseImageData::onDeploy(%item, %plyr, %slot)
   {
      Parent::onDeploy(%item, %plyr, %slot);
      // ZOD 1-29-03: Undeploy code - BadShot/ZOD
      %deplObj.justdeployed = 1;
      schedule(2500, %deplObj, "resetjustdeployed", %deplObj);
   }

   function ServerCmdStartUseBackpack(%client, %data) 
   {
      // ZOD 1-29-03: Undeployment of deployed objects - BadShot
      if((%client.player.getMountedImage($BackpackSlot) == 0) && (%client.player.thrownChargeId == 0))
   	   pickupDeployable(%client);
      else
         Parent::ServerCmdStartUseBackpack(%client, %data);
   }
};

function CTFGame::countFlips(%game)
{
   return false;
}

function resetjustdeployed(%obj)
{
   %obj.justdeployed = 0;
}

function pickupDeployable(%client)
{
   %player = %client.player;
   if(%player.inStation)
      return;

   %Masks = $TypeMasks::StaticShapeObjectType | $TypeMasks::TurretObjectType;
   %eyeVec = VectorNormalize(%player.getEyeVector());
   %srchRange = VectorScale(%eyeVec, 5.0);
   %plTm = %player.getEyeTransform();
   %plyrLoc = firstWord(%plTm) @ " " @ getWord(%plTm, 1) @ " " @ getWord(%plTm, 2);
   %srchEnd = VectorAdd(%plyrLoc, %srchRange);
   %potDep = ContainerRayCast(%player.getEyeTransform(), %srchEnd, %Masks);
   if(%potDep)
   {
      %item = %potDep.getDataBlock().item;
      if(%item !$= "" && %potDep.getDataBlock().deployedObject == true)
      {
         if(%potDep.getDamageLevel() < 0.5 && !%potDep.isDisabled())
         {
            if(%player.maxInventory(%item) > 0)
            {
               if(%potDep.team == %player.team)
               {
                  if(!%potDep.justdeployed)
                  {
                     switch$ ( %item ) // ZOD 1-7-03: Special case for certain turrets etc.
                     {
                        case "TurretOutdoorDeployable" or "TurretIndoorDeployable":
                           %potDep.justdeployed = true;
                           if(isObject(%potDep.lastProjectile))
                              %potDep.lastProjectile.delete();

                           %potDep.clearSelfPowered();
                           $TeamDeployedCount[%player.team, %item]--;
                           %potDep.schedule(250, "delete");

                        case "MotionSensorDeployable" or "PulseSensorDeployable":
                           %potDep.justdeployed = true;
                           %player.deploySensors--;
                           %client.updateSensorPackText(%player.deploySensors);
                           %potDep.schedule(100, "delete");
                           $TeamDeployedCount[%player.team, %item]--;

                        case "InventoryDeployable":
                           %potDep.justdeployed = true;
                           %potDep.trigger.delete();
                           %potDep.schedule(100, "delete");
                           $TeamDeployedCount[%player.team, %item]--;

                        default:
                           %potDep.justdeployed = true;
                           %potDep.schedule(100, "delete");
                           $TeamDeployedCount[%player.team, %item]--;
                     }
                     %npack = new Item() {
                        dataBlock = %item;
                        sensors = 1;
                     };
                     MissionCleanup.add(%npack);
                     %pos = %potDep.getPosition();
                     %npack.setTransform(VectorAdd(%pos, "0 0 .75") SPC "0 0 1" SPC (getRandom() * 360));
                     %npack.schedulePop(); // ZOD: Really important, otherwise the item lingers on the map forever or until its picked up.
                  }
                  else
                     messageClient(%client, 'MsgJustDeployed', '\c0Object was just deployed, please wait a moment.');
               }
               else
                  messageClient(%client, 'MsgWrongTeam', '\c0Access Denied. Wrong Team.');
            }
            else
               messageClient(%client, 'MsgTooSmall', '\c0You can\'t undeploy this object in your armor.');
         }
         else
            messageClient(%client, 'MsgDisabled', '\c0Object is to heavily damaged.');
      }
      else
         messageClient(%client, 'MsgNotDeployable', '\c0Object is not undeployable.');
   }
   else
      messageClient(%client, 'MsgNothing', '\c0No undeployable in sight.');
}

function orgRandom(%organicName, %num, %radius, %startPos)
{
   %spacing = 5.0;
   %organicIndex = -1;
   for (%i = 0; %i < $NumStaticTSObjects; %i++)
   {
      if (getWord($StaticTSObjects[%i], 1) $= %organicName)
      {
         %organicIndex = %i;
         break;
      }
   }
   %shapeFileName = getWord($StaticTSObjects[%organicIndex], 2);
   %maxSlope = getWord($StaticTSObjects[%organicIndex], 3);
   if (%maxSlope $= "")
      %maxSlope = 40;

   %zOffset = getWord($StaticTSObjects[%organicIndex], 4);
   if (%zOffset $= "")
      %zOffset = 0;

   %slopeWithTerrain = getWord($StaticTSObjects[%organicIndex], 5);
   if (%slopeWithTerrain $= "")
      %slopeWithTerrain = false;

   %minScale = getWord($StaticTSObjects[%organicIndex], 6);
   %maxScale = getWord($StaticTSObjects[%organicIndex], 7);

   //set up folders in mis file
   $RandomOrganicsAdded++;
   if(!isObject(RandomOrganics))
   {
      %randomOrgGroup = new simGroup(RandomOrganics);
      MissionGroup.add(%randomOrgGroup);
   }
   %groupName = "Addition"@$RandomOrganicsAdded@%organicName;
   %group = new simGroup(%groupName);
   RandomOrganics.add(%group);

   if(%startPos !$= "" && %startPos != -1)
      %ctr = %startPos;
   else if(isObject(LocalClientConnection.camera))
      %ctr = LocalClientConnection.camera.getPosition();
   else
      %ctr = NameToID("MissionGroup/MissionArea").getPosition();

   %areaX = getWord(%ctr, 0) - %radius;
   %areaY = getWord(%ctr, 1) - %radius;
   
   %orgCount = %num;
   while((%orgCount > 0) && (%retries < (15000 / %maxSlope)))
   {
      %x = (getRandom(mFloor(%areaX / 8), mFloor((%areaX + (%radius * 2)) / 8)) * 8) + 4;  //tile center             
      %y = (getRandom(mFloor(%areaY / 8), mFloor((%areaY + (%radius * 2)) / 8)) * 8) + 4;    

      %start = %x @ " " @ %y @ " 2000";
      %end = %x @ " " @ %y @ " -1";
      %ground = containerRayCast(%start, %end, $TypeMasks::TerrainObjectType, 0);      
      %z = getWord(%ground, 3);
      %z += %zOffset;            
      %position = %x @ " " @ %y @ " " @ %z;
     
      %start = %x + 2 @ " " @ %y @ " 2000";
      %end = %x + 2 @ " " @ %y @ " -1";
      %hit1 = containerRayCast(%start, %end, $TypeMasks::TerrainObjectType, 0);
            
      %start = %x - 2 @ " " @ %y @ " 2000";
      %end = %x - 2 @ " " @ %y @ " -1";
      %hit2 = containerRayCast(%start, %end, $TypeMasks::TerrainObjectType, 0);    

      %norm1 = getWord(%hit1, 4) @ " " @ getWord(%hit1, 5) @ " " @ getWord(%hit1, 6);
      %norm2 = getWord(%hit2, 4) @ " " @ getWord(%hit2, 5) @ " " @ getWord(%hit2, 6);

      %angNorm1 = getTerrainAngle(%norm1);
      %angNorm2 = getTerrainAngle(%norm2);
      if ((getTerrainAngle(%norm1) > %maxSlope) || (getTerrainAngle(%norm2) > %maxslope))
      {      
         %retries++;
         continue;
      }

      %terrainNormal = VectorAdd(%norm1, %norm2);
      %terrainNormal = VectorNormalize(%terrainNormal);       
    
      InitContainerRadiusSearch(%position, %spacing,  $TypeMasks::VehicleObjectType | 
                                $TypeMasks::MoveableObjectType | $TypeMasks::StaticShapeObjectType |
                                $TypeMasks::StaticTSObjectType | $TypeMasks::ForceFieldObjectType |
                                $TypeMasks::TurretObjectType | $TypeMasks::InteriorObjectType | 
                                $TypeMasks::ItemObjectType);

      %this = containerSearchNext();
      if(%this)
      {        
         %retries++;
         continue;         
      }
 
      if(%slopeWithTerrain)
      {
         %rotAxis = vectorCross(%terrainNormal, "0 0 1");
         %rotAxis = vectorNormalize(%rotAxis);
         %rotation = %rotAxis @ " " @ getTerrainAngle(%terrainNormal);              
      }        
      else
         %rotation = "1 0 0 0";

      %randomAngle = getRandom(360);
      %zrot = MatrixCreate("0 0 0", "0 0 1 " @ %randomAngle);
      %orient = MatrixCreate(%position, %rotation);
      %finalXForm = MatrixMultiply(%orient, %zrot);
      %scaleMin = 8;
      %scaleMax = 14;
      if(%minScale)
         %scaleMin = %minScale * 10;

      if(%maxScale)
         %scaleMax = %maxScale * 10;

      %scaleInt = getRandom(%scaleMin, %scaleMax);
      %scale = %scaleInt/10;
      
      %newOrganic = new TSStatic() {
         position  = %x SPC %y SPC (%z += %zoffset);
         rotation  = %rotation;
         scale     = %scale SPC %scale SPC %scale;
         shapeName = %shapeFileName;
      };
      %group.add(%newOrganic);
      %newOrganic.setTransform(%finalXForm);

      %orgCount--;
      %retries = 0;
   }
   if (%orgCount > 0)
   {
      error("Unable to place all shapes, area saturated.");
      error("Looking for clear area " @ (%spacing * 2) @ " meters in diameter, with a max slope of " @ %maxSlope);
   }
   echo("Placed " @ %num - %orgCount @ " of " @ %num);
}
